@@ -1,8 +1,8 @@
 # Vend::Cart - Interchange shopping cart management routines
 #
-# $Id: Cart.pm,v 2.10 2003/11/17 14:24:11 mheins Exp $
+# $Id: Cart.pm,v 2.13 2005/11/08 18:14:44 jon Exp $
 #
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -25,7 +25,7 @@
 
 package Vend::Cart;
 
-$VERSION = substr(q$Revision: 2.10 $, 10);
+$VERSION = substr(q$Revision: 2.13 $, 10);
 
 use strict;
 
@@ -49,6 +49,7 @@ sub STORE {
 	if( ref($cart) eq 'ARRAY' ) {
 		for(keys %$::Carts) {
 #::logDebug("checking name $_ via ref comparison");
+			next unless $::Carts->{$_};
 			$name = $_ if $::Carts->{$_} eq $cart;
 		}
 
@@ -179,6 +180,11 @@ sub toss_cart {
 	my $sub;
 	my (@master);
 	my (@cascade);
+	my ($raise_event, $quantity_raise_event)
+		= @{$Vend::Cfg}{qw/CartTrigger CartTriggerQuantity/};
+	$quantity_raise_event = $raise_event && $quantity_raise_event;
+	my $event_cartname = $cartname || $Vend::CurrentCart;
+	my $old_item;
 	DELETE: for (;;) {
 		foreach $i (0 .. $#$s) {
 			my $item = $s->[$i];
@@ -194,7 +200,13 @@ sub toss_cart {
 				elsif ( $item->{mv_ci} ) {
 					push (@master, $item->{mv_ci});
 				}
+				$old_item = $s->[$i] if $raise_event;
 				splice(@$s, $i, 1);
+				trigger_delete(
+						$s,
+						$old_item,
+						$event_cartname
+					) if $raise_event;
 				next DELETE;
 			}
 
@@ -215,8 +227,15 @@ sub toss_cart {
 					$item->{quantity} < $item->{mv_min_quantity}
 					)
 				{
+					$old_item = { %$item } if $quantity_raise_event;
 					$item->{quantity} = $item->{mv_min_quantity};
 					$item->{mv_min_under} = 1;
+					trigger_update(
+							$s,
+							$item,
+							$old_item,
+							$event_cartname
+						) if $quantity_raise_event;
 				}
 			}
 
@@ -234,18 +253,25 @@ sub toss_cart {
 					$item->{quantity} > $item->{mv_max_quantity}
 					)
 				{
+					$old_item = { %$item } if $quantity_raise_event;
 					$item->{quantity} = $item->{mv_max_quantity};
 					$item->{mv_max_over} = 1;
+					trigger_update(
+							$s,
+							$item,
+							$old_item,
+							$event_cartname
+						) if $quantity_raise_event;
 				}
 			}
 
-			next unless $Vend::Cfg->{Limit}{cart_quantity_per_line};
+			next unless $Vend::Cfg->{Limit}{cart_quantity_per_line}
+				and $item->{quantity} > $Vend::Cfg->{Limit}{cart_quantity_per_line};
 			
-			$item->{quantity} = $Vend::Cfg->{Limit}{cart_quantity_per_line}
-				if
-					$item->{quantity}
-						>
-					$Vend::Cfg->{Limit}{cart_quantity_per_line};
+			$old_item = { %$item } if $quantity_raise_event;				
+			$item->{quantity} = $Vend::Cfg->{Limit}{cart_quantity_per_line};
+			trigger_update( $s, $item, $old_item, $event_cartname )
+				if $quantity_raise_event;
 		}
 		last DELETE;
 	}
@@ -274,6 +300,10 @@ sub toss_cart {
 			}
 			@items = @$s;
 			@{$s} = @items[sort {$a <=> $b} keys %save];
+			if ($raise_event and scalar(@items) > scalar(@$s)) {
+				trigger_delete($s, $items[$_], $event_cartname)
+					for grep { ! $save{$_} } (0..$#items);
+			}
 		}
 	}
 	Vend::Interpolate::levies(1, $cartname);
@@ -293,6 +323,47 @@ sub toss_cart {
 
 =cut
 
+sub trigger_event {
+	my($s, $action, $new_row, $old_row, $cartname) = @_;
+	return unless my $subs = $Vend::Cfg->{CartTrigger};
+	$subs = [ split /\s+/, $subs ] unless ref $subs eq 'ARRAY';
+	my @results;
+	for my $subname (@$subs) {
+		next unless my $sub
+			= $Vend::Cfg->{Sub}{$subname}
+			|| $Global::GlobalSub->{$subname};
+
+		my $result;
+		eval {
+			$result = $sub->($s, $action, $new_row, $old_row, $cartname);
+		};
+		if ($@) {
+			::logError( "CartTrigger event handler '%' action '%' returned error:\n%",
+				$Vend::Cfg->{CartTrigger},
+				$action,
+				$@ );
+			$result = undef;
+		}
+		push @results, $result
+	}
+	return @results;
+}
+
+sub trigger_add {
+	my($s,$new_row,$cartname) = @_;
+	return trigger_event($s, 'add', $new_row, undef, $cartname);
+}
+
+sub trigger_update {
+	my($s,$new_row,$old_row,$cartname) = @_;
+	return trigger_event($s, 'update', $new_row, $old_row, $cartname);
+}
+
+sub trigger_delete {
+	my($s,$old_row,$cartname) = @_;
+	return trigger_event($s, 'delete', undef, $old_row, $cartname);
+}
+
 1;
 
 __END__
