@@ -1,8 +1,8 @@
 # Vend::Form - Generate Form widgets
 # 
-# $Id: Form.pm,v 2.43 2004/02/22 19:28:37 mheins Exp $
+# $Id: Form.pm,v 2.64.2.1 2005/11/30 15:01:56 racke Exp $
 #
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -31,14 +31,15 @@ use Vend::Interpolate;
 use Vend::Util;
 use Vend::Tags;
 use strict;
+no warnings qw(uninitialized numeric);
 use POSIX qw{strftime};
 
-use vars qw/@ISA @EXPORT @EXPORT_OK $VERSION %Template/;
+use vars qw/@ISA @EXPORT @EXPORT_OK $VERSION %Template %ExtraMeta/;
 
 require Exporter;
 @ISA = qw(Exporter);
 
-$VERSION = substr(q$Revision: 2.43 $, 10);
+$VERSION = substr(q$Revision: 2.64.2.1 $, 10);
 
 @EXPORT = qw (
 	display
@@ -74,14 +75,12 @@ my $Tag = new Vend::Tags;
 		.
 		qq({ROWS?} size="{ROWS}"{/ROWS?})
 		.
-		qq({DISABLED?} DISABLED{/DISABLED?})
+		qq({DISABLED?} disabled{/DISABLED?})
 		.
-		qq({MULTIPLE?} MULTIPLE{/MULTIPLE?})
+		qq({MULTIPLE?} multiple{/MULTIPLE?})
 		.
 		qq({EXTRA?} {EXTRA}{/EXTRA?})
 		.
-		qq({JS?} {JS}{/JS?})
-		.
 		qq(>)
 		,
 	selecttail =>
@@ -96,10 +95,12 @@ my $Tag = new Vend::Tags;
 		.
 		qq({COLS?} cols="{COLS}"{/COLS?})
 		.
-		qq({DISABLED?} DISABLED{/DISABLED?})
+		qq({DISABLED?} disabled{/DISABLED?})
 		.
 		qq({MAXLENGTH?} maxlength="{MAXLENGTH}"{/MAXLENGTH?})
 		.
+		qq({TTITLE?} title="{TTITLE}"{/TTITLE?})
+		.
 		qq({WRAP?} wrap="{WRAP}"{/WRAP?})
 		.
 		qq({EXTRA?} {EXTRA}{/EXTRA?})
@@ -122,6 +123,8 @@ my $Tag = new Vend::Tags;
 	file =>
 		qq({PREPEND}<input type="file" name="{NAME}" value="{ENCODED}")
 		.
+		qq({TTITLE?} title="{TTITLE}"{/TTITLE?})
+		.
 		qq({COLS?} size="{COLS}"{/COLS?})
 		.
 		qq({EXTRA?} {EXTRA}{/EXTRA?})
@@ -131,21 +134,27 @@ my $Tag = new Vend::Tags;
 	filetext =>
 		qq({PREPEND}<input type="file" name="{NAME}" value="{ENCODED}")
 		.
+		qq({TTITLE?} title="{TTITLE}"{/TTITLE?})
+		.
 		qq({COLS?} size="{COLS}"{/COLS?})
 		.
 		qq({EXTRA?} {EXTRA}{/EXTRA?})
 		.
-		qq(><br><textarea cols="{WIDTH}" rows="{HEIGHT}" NAME="{NAME}">{ENCODED}</textarea>{APPEND})
+		qq(><br{XTRAILER}><textarea cols="{WIDTH}" rows="{HEIGHT}" name="{NAME}">{ENCODED}</textarea>{APPEND})
 		,
 	text =>
 		qq({PREPEND}<input type="text" name="{NAME}" value="{ENCODED}")
 		.
 		qq({COLS?} size="{COLS}"{/COLS?})
 		.
-		qq({DISABLED?} DISABLED{/DISABLED?})
+		qq({TTITLE?} title="{TTITLE}"{/TTITLE?})
+		.
+		qq({DISABLED?} disabled{/DISABLED?})
 		.
 		qq({MAXLENGTH?} maxlength="{MAXLENGTH}"{/MAXLENGTH?})
 		.
+		qq({TTITLE?} title="{TTITLE}"{/TTITLE?})
+		.
 		qq({EXTRA?} {EXTRA}{/EXTRA?})
 		.
 		qq(>{APPEND})
@@ -162,32 +171,36 @@ my $Tag = new Vend::Tags;
 		.
 		qq({EXTRA?} {EXTRA}{/EXTRA?})
 		.
-		qq(>{ENCODED}{APPEND})
+		qq(>{FILTERED?}{FILTERED}{/FILTERED?}{FILTERED:}{ENCODED}{/FILTERED:}{APPEND})
 		,
 	boxstd =>
 		qq(<input type="{VARIANT}" name="{NAME}" value="{TVALUE}")
 		.
 		qq({EXTRA?} {EXTRA}{/EXTRA?})
 		.
-		qq({DISABLED?} DISABLED{/DISABLED?})
+		qq({TTITLE?} title="{TTITLE}"{/TTITLE?})
 		.
-		qq({SELECTED?} CHECKED{/SELECTED?})
+		qq({DISABLED?} disabled{/DISABLED?})
 		.
-		qq(>&nbsp;{TLABEL})
+		qq({SELECTED?} checked{/SELECTED?})
+		.
+		qq(>&nbsp;{TTITLE?}<span title="{TTITLE}">{/TTITLE?}{TLABEL}{TTITLE?}</span>{/TTITLE?})
 		,
 	boxnbsp =>
 		qq(<input type="{VARIANT}" name="{NAME}" value="{TVALUE}")
 		.
+		qq({TTITLE?} title="{TTITLE}"{/TTITLE?})
+		.
 		qq({EXTRA?} {EXTRA}{/EXTRA?})
 		.
-		qq({DISABLED?} DISABLED{/DISABLED?})
+		qq({DISABLED?} disabled{/DISABLED?})
 		.
-		qq({SELECTED?} CHECKED{/SELECTED?})
+		qq({SELECTED?} checked{/SELECTED?})
 		.
-		qq(>&nbsp;{TLABEL}&nbsp;&nbsp;)
+		qq(>&nbsp;{TTITLE?}<span title="{TTITLE}">{/TTITLE?}{TLABEL}{TTITLE?}</span>{/TTITLE?}&nbsp;&nbsp;)
 		,
 	boxlabel =>
-		qq(<td{TD_LABEL?} {TD_LABEL}{/TD_LABEL?}>)
+		qq(<td{TD_LABEL?} {TD_LABEL}{/TD_LABEL?}{TTITLE?} title="{TTITLE}"{/TTITLE?}>)
 		.
 		qq({FONT?}<font size="{FONT}">{/FONT?})
 		.
@@ -200,18 +213,20 @@ my $Tag = new Vend::Tags;
 		.
 		qq(<input type="{VARIANT}" name="{NAME}" value="{TVALUE}")
 		.
-		qq({DISABLED?} DISABLED{/DISABLED?})
+		qq({TTITLE?} title="{TTITLE}"{/TTITLE?})
+		.
+		qq({DISABLED?} disabled{/DISABLED?})
 		.
 		qq({EXTRA?} {EXTRA}{/EXTRA?})
 		.
-		qq({SELECTED?} CHECKED{/SELECTED?})
+		qq({SELECTED?} checked{/SELECTED?})
 		.
 		qq(>)
 		.
 		qq(</td>)
 		,
 	boxgroup =>
-		qq(</tr><tr><td{TD_GROUP?} {TD_GROUP}{/TD_GROUP?} COLSPAN=2>)
+		qq(</tr><tr><td{TD_GROUP?} {TD_GROUP}{/TD_GROUP?} colspan="2">)
 		.
 		qq(<b>{TVALUE}</b>)
 		.
@@ -224,6 +239,7 @@ $Template{default} = $Template{text};
 sub attr_list {
 	my ($body, $hash) = @_;
 	return $body unless ref($hash) eq 'HASH';
+
 	$body =~ s!\{([A-Z_]+)\}!$hash->{lc $1}!g;
 	$body =~ s!\{([A-Z_]+)\|($Some)\}!$hash->{lc $1} || $2!eg;
 	$body =~ s!\{([A-Z_]+)\s+($Some)\}! $hash->{lc $1} ? $2 : ''!eg;
@@ -288,20 +304,19 @@ sub current_label {
 
 sub links {
 	my($opt, $opts) = @_;
-#warn "called links opts=$opts\n";
 
-	$opt->{joiner} = Vend::Interpolate::get_joiner($opt->{joiner}, "<BR>");
+	$opt->{joiner} = Vend::Interpolate::get_joiner($opt->{joiner}, "<br$Vend::Xtrailer>");
 	my $name = $opt->{name};
 	my $default = defined $opt->{value} ? $opt->{value} : $opt->{default};
 
 	$opt->{extra} = " $opt->{extra}" if $opt->{extra};
 
 	my $template = $opt->{template} || <<EOF;
-<A HREF="{URL}"{EXTRA}>{SELECTED <B>}{LABEL}{SELECTED </B>}</A>
+<a href="{URL}"{EXTRA}>{SELECTED <b>}{LABEL}{SELECTED </b>}</a>
 EOF
 
 	my $o_template = $opt->{o_template} || <<EOF;
-<B>{TVALUE}</B>
+<b>{TVALUE}</b>
 EOF
 
 	my $href = $opt->{href} || $Global::Variable->{MV_PAGE};
@@ -338,9 +353,11 @@ EOF
 
 		my $form = $opt->{form};
 
+		$attr->{label} =~ s/\s/&nbsp;/g if $opt->{nbsp};
+
 		$attr->{url} = Vend::Interpolate::tag_area(
 						$href,
-						'',
+						undef,
 						{
 							form => "$name=$attr->{value}\n$opt->{form}",
 							secure => $opt->{secure},
@@ -424,42 +441,62 @@ sub date_widget {
 		$now += $sign eq '+' ? $adjust : -$adjust;
 	}
 
+	my $sel_extra;
+	my $opt_extra;
+	for(qw/ class style extra /) {
+		my $stag = "select_$_";
+		my $otag = "option_$_";
+		my $selapp;
+		my $optapp;
+
+		if($_ eq 'extra') {
+			$selapp = " $opt->{$stag}";
+			$optapp = " $opt->{$otag}";
+		}
+		else {
+			$selapp = qq{ $_="$opt->{$stag}"};
+			$optapp = qq{ $_="$opt->{$otag}"};
+		}
+		$sel_extra .= $opt->{$stag} ? $selapp : '';
+		$opt_extra .= $opt->{$otag} ? $optapp : '';
+	}
+
 	my @t = localtime($now || time);
-	if (not $val) {
+	my $sel = 0;
+	my $out = qq{<select name="$name"$sel_extra>};
+	my $o;
+	if ($opt->{blank}) {
+		$out .= qq{<option value="0"$opt_extra>------</option>};
+	} elsif (not $val) {
+		# use current time with possible adjustments as default value
 		$t[2]++ if $t[2] < 23;
 		$val = POSIX::strftime("%Y%m%d%H00", @t);
 	}
-	my $sel = 0;
-	my $out = qq{<SELECT NAME="$name">};
-	my $o;
-	if ($opt->{blank}) {
-		$out .= '<OPTION VALUE="0">------</OPTION>';
-	}
 	for(@Months) {
-		$o = qq{<OPTION VALUE="$_->[0]">} . errmsg($_->[1]) . '</OPTION>';
+		$o = qq{<option value="$_->[0]"$opt_extra>} . errmsg($_->[1]) . '</option>';
 		($out .= $o, next) unless ! $sel and $val;
 		$o =~ s/>/ SELECTED>/ && $sel++
 			if substr($val, 4, 2) eq $_->[0];
 		$out .= $o;
 	}
 	$sel = 0;
-	$out .= qq{</SELECT>};
-	$out .= qq{<INPUT TYPE=hidden NAME="$name" VALUE="/">};
-	$out .= qq{<SELECT NAME="$name">};
+	$out .= qq{</select>};
+	$out .= qq{<input type="hidden" name="$name" value="/">};
+	$out .= qq{<select name="$name"$sel_extra>};
 	if ($opt->{blank}) {
-		$out .= '<OPTION VALUE="0">--</OPTION>';
+		$out .= qq{<option value="0"$opt_extra>--</option>};
 	}
 	for(@Days) {
-		$o = qq{<OPTION VALUE="$_->[0]">$_->[1]} . '</OPTION>';
+		$o = qq{<option value="$_->[0]"$opt_extra>$_->[1]} . '</option>';
 		($out .= $o, next) unless ! $sel and $val;
 		$o =~ s/>/ SELECTED>/ && $sel++
 			if substr($val, 6, 2) eq $_->[0];
 		$out .= $o;
 	}
 	$sel = 0;
-	$out .= qq{</SELECT>};
-	$out .= qq{<INPUT TYPE=hidden NAME="$name" VALUE="/">};
-	$out .= qq{<SELECT NAME="$name">};
+	$out .= qq{</select>};
+	$out .= qq{<input type="hidden" name="$name" value="/">};
+	$out .= qq{<select name="$name"$sel_extra>};
 	if(my $by = $opt->{year_begin} || $::Variable->{UI_DATE_BEGIN}) {
 		my $cy = $t[5] + 1900;
 		my $ey = $opt->{year_end}  || $::Variable->{UI_DATE_END} || ($cy + 10);
@@ -472,25 +509,25 @@ sub date_widget {
 		@Years = ($by .. $ey);
 	}
 	if ($opt->{blank}) {
-		$out .= '<OPTION VALUE="0000">----</OPTION>';
+		$out .= qq{<option value="0000"$opt_extra>----</option>};
 	}
 	for(@Years) {
-		$o = qq{<OPTION>$_} . '</OPTION>';
+		$o = qq{<option$opt_extra>$_} . '</option>';
 		($out .= $o, next) unless ! $sel and $val;
 		$o =~ s/>/ SELECTED>/ && $sel++
 			if substr($val, 0, 4) eq $_;
 		$out .= $o;
 	}
-	$out .= qq{</SELECT>};
+	$out .= qq{</select>};
 	return $out unless $opt->{time};
 
 	$val =~ s/^\d{8}//;
 	$val =~ s/\D+//g;
 	$val = round_to_fifteen($val);
-	$out .= qq{<INPUT TYPE=hidden NAME="$name" VALUE=":">};
-	$out .= qq{<SELECT NAME="$name">};
+	$out .= qq{<input type="hidden" name="$name" value=":">};
+	$out .= qq{<select name="$name"$sel_extra>};
 	if ($opt->{blank}) {
-		$out .= '<OPTION VALUE="0">--:--</OPTION>';
+		$out .= qq{<option value="0"$opt_extra>--:--</option>};
 	}
 	
 	my $ampm = defined $opt->{ampm} ? $opt->{ampm} : 1;
@@ -498,8 +535,40 @@ sub date_widget {
 	undef $sel;
 	my %special = qw/ 0 midnight 12 noon /;
 	
-	for my $hr ( 0 .. 23) {
-		for my $min ( 0,15,30,45 ) {
+	my @min;
+
+	$opt->{minutes} ||= '';
+
+	if($opt->{minutes} =~ /half/i) {
+		@min = (0,30);
+	}
+	elsif($opt->{minutes} =~ /hourly/i) {
+		@min = (0);
+	}
+	elsif($opt->{minutes} =~ /ten/i) {
+		@min = (0,10,20,30,40,50);
+	}
+	elsif($opt->{minutes} =~ /[\0,]/) {
+		@min = grep /^\d+$/ && $_ <= 59, split /[\0,\s]+/, $opt->{minutes};
+	}
+	else {
+		@min = (0,15,30,45);
+	}
+
+	$opt->{start_hour} ||= 0;
+	for(qw/start_hour end_hour/) {
+		$opt->{$_} = int(abs($opt->{$_}));
+		if($opt->{$_} > 23) {
+			$opt->{$_} = 0;
+		}
+	}
+	$opt->{start_hour}	||= 0;
+	$opt->{end_hour}	||= 23;
+	
+	for my $hr ( $opt->{start_hour} .. $opt->{end_hour} ) {
+		next if defined $opt->{start_hour} and $hr < $opt->{start_hour};
+		next if defined $opt->{end_hour} and $hr > $opt->{end_hour};
+		for my $min ( @min ) {
 			my $disp_hour = $hr;
 			if($opt->{ampm}) {
 				if( $hr < 12) {
@@ -522,7 +591,7 @@ sub date_widget {
 				$disp_hour = sprintf("%02d:%02d", $hr, $min);
 			}
 			my $time = sprintf "%02d%02d", $hr, $min;
-			$o = sprintf qq{<OPTION VALUE="%s">%s}, $time, $disp_hour;
+			$o = sprintf qq{<option value="%s"$opt_extra>%s}, $time, $disp_hour;
 			($out .= $o, next) unless ! $sel and $val;
 #::logDebug("prospect=$time actual=$val");
 			$o =~ s/>/ SELECTED>/ && $sel++
@@ -530,7 +599,7 @@ sub date_widget {
 			$out .= $o;
 		}
 	}
-	$out .= "</SELECT>";
+	$out .= "</select>";
 	return $out;
 }
 
@@ -541,7 +610,7 @@ sub option_widget_box {
 	$val =~ s/"/&quot;/g;
 	$lab =~ s/"/&quot;/g;
 	$width = 10 if ! $width;
-	return qq{<TR><TD><SMALL><INPUT TYPE=text NAME="$name" VALUE="$val" SIZE=$half></SMALL></TD><TD><SMALL><INPUT TYPE=text NAME="$name" VALUE="$lab" SIZE=$width></SMALL></TD><TD><SMALL><SMALL><SELECT NAME="$name"><OPTION value="0">no<OPTION value="1"$sel>default*</SELECT></SMALL></SMALL></TD></TR>};
+	return qq{<tr><td><small><input type="text" name="$name" value="$val" size="$half"></small></td><td><small><input type="text" name="$name" value="$lab" size="$width"></small></td><td><small><select name="$name"><option value="0">no<option value="1"$sel>default*</select></small></td></tr>};
 }
 
 sub option_widget {
@@ -553,7 +622,13 @@ sub option_widget {
 		unless length($opt->{filter});
 	$val = Vend::Interpolate::filter_value($opt->{filter}, $val);
 	my @opts = split /\s*,\s*/, $val;
-	my $out = "<TABLE CELLPADDING=0 CELLSPACING=0><TR><TH><SMALL>Value</SMALL></TH><TH ALIGN=LEFT COLSPAN=2><SMALL>Label</SMALL></TH></TR>";
+
+	my $out = qq{<table cellpadding="0" cellspacing="0"><tr><th><small>};
+	$out .= errmsg('Value');
+	$out .= qq{</small></th><th align="left" colspan="2"><small>};
+	$out .= errmsg('Label');
+	$out .= qq{</small></th></tr>};
+
 	my $done;
 	my $height = $opt->{height} || 5;
 	$height -= 2;
@@ -571,7 +646,7 @@ sub option_widget {
 	}
 	$out .= option_widget_box($name, '', '', '', $width);
 	$out .= option_widget_box($name, '', '', '', $width);
-	$out .= "</TABLE>";
+	$out .= "</table>";
 }
 
 
@@ -579,9 +654,14 @@ sub movecombo {
 	my ($opt, $opts) = @_;
 	my $name = $opt->{name};
 	$opt->{name} = "X$name";
-	my $ejs = ",1" if $opt->{rows} > 1;
-	$opt->{extra} .= qq{ onChange="addItem(this.form['X$name'],this.form['$name']$ejs)"}
+	my $usenl = $opt->{rows} > 1 ? 1 : 0;
+	my $only = $opt->{replace} ? 1 : 0;
+	$opt->{extra} .= qq{ onChange="addItem(this.form['X$name'],this.form['$name'],$usenl,$only)"}
             unless $opt->{extra} =~ m/\bonchange\s*=/i;
+
+	$opt->{rows} = $opt->{height} unless length($opt->{rows});
+	$opt->{cols} = $opt->{width} unless length($opt->{cols});
+
 	my $tbox = '';
 	my $out = dropdown($opt, $opts);
 
@@ -592,7 +672,7 @@ sub movecombo {
 			$template .= q( cols="{COLS|20}" name="{NAME}">{ENCODED}</textarea>);
 		}
 		else {
-			$template .= qq(<input TYPE="text" size="{COLS||40}");
+			$template .= qq(<input type="text" size="{COLS||40}");
 			$template .= qq( name="{NAME}" value="{ENCODED}">);
 		}
 	}
@@ -608,7 +688,7 @@ sub combo {
 	if($opt->{textarea}) {
 		my $template = $opt->{o_template};
 		if(! $template) {
-			$template = '<br>';
+			$template = "<br$Vend::Xtrailer>";
 			if(! $opt->{rows} or $opt->{rows} > 1) {
 				$template .= q(<textarea rows="{ROWS|2}" wrap="{WRAP|virtual}");
 				$template .= q( cols="{COLS|60}" name="{NAME}">);
@@ -617,7 +697,7 @@ sub combo {
 				$template .= q(</textarea>);
 			}
 			else {
-				$template .= qq(<input TYPE="text" size="{COLS|40}");
+				$template .= qq(<input type="text" size="{COLS|40}");
 				$template .= qq( name="{NAME}" value=");
 				$template .= '{ENCODED}'
 					unless $opt->{conditional_text} and length($opt->{value}) < 3;
@@ -627,8 +707,8 @@ sub combo {
 		$addl = attr_list($template, $opt);
 	}
 	else {
-		$addl = qq|<INPUT TYPE=text NAME="$opt->{name}"|;
-		$addl   .= qq| SIZE="$opt->{cols}" VALUE="">|;
+		$addl = qq|<input type="text" name="$opt->{name}"|;
+		$addl   .= qq| size="$opt->{cols}" value="">|;
 	}
 	if($opt->{reverse}) {
 		$opt->{append} = length($opt->{append}) ? "$addl$opt->{append}" : $addl;
@@ -689,8 +769,9 @@ sub dropdown {
 	my $no_encode = $opt->{pre_filter} eq 'decode_entities' ? 1 : 0;
 	
 	for(@$opts) {
-		my ($value, $label) = @$_;
+		my ($value, $label, $help) = @$_;
 		encode($label, $ESCAPE_CHARS::std) unless $no_encode;
+		encode($help, $ESCAPE_CHARS::std) if $help;
 		if($value =~ /^\s*\~\~(.*)\~\~\s*$/) {
 			my $label = $1;
 			if($optgroup_one++) {
@@ -709,9 +790,7 @@ sub dropdown {
 			$value =~ s/\*$// and $select = 1;
 		}
 
-		if (defined $default) {
-			$select = '';
-		}
+		$select = '' if defined $default;
 
 		my $extra = '';
 		my $attr = {};
@@ -726,6 +805,7 @@ sub dropdown {
 		my $vvalue = $value;
 		encode($vvalue, $ESCAPE_CHARS::std);
 		$run .= qq| value="$vvalue"|;
+		$run .= qq| title="$help"| if $help;
 		if (length($default)) {
 			$regex	= qr/$re_b\Q$value\E$re_e/;
 			$default =~ $regex and $select = 1;
@@ -827,27 +907,27 @@ sub box {
 		$template = $Template{boxnbsp};
 	}
 	elsif ($opt->{left}) {
-		$header = '<TABLE>';
-		$footer = '</TABLE>';
-		$template = '<TR>' unless $inc;
+		$header = '<table>';
+		$footer = '</table>';
+		$template = '<tr>' unless $inc;
 		$template .= $Template{boxvalue};
 		$template .= $Template{boxlabel};
-		$template .= '</TR>' unless $inc;
+		$template .= '</tr>' unless $inc;
 		$o_template = $Template{boxgroup};
 	}
 	elsif ($opt->{right}) {
-		$header = '<TABLE>';
-		$footer = '</TABLE>';
-		$template = '<TR>' unless $inc;
+		$header = '<table>';
+		$footer = '</table>';
+		$template = '<tr>' unless $inc;
 		$template .= $Template{boxlabel};
 		$template .= $Template{boxvalue};
-		$template .= '</TR>' unless $inc;
+		$template .= '</tr>' unless $inc;
 		$o_template = $Template{boxgroup};
 	}
 	else {
 		$template = $Template{boxstd};
 	}
-	$o_template ||= '<BR><b>{TVALUE}</b><BR>';
+	$o_template ||= "<br$Vend::Xtrailer><b>{TVALUE}</b><br$Vend::Xtrailer>";
 
 	my $run = $header;
 
@@ -858,8 +938,9 @@ sub box {
 	my $no_encode = $opt->{pre_filter} eq 'decode_entities' ? 1 : 0;
 
 	for(@$opts) {
-		my($value,$label) = @$_;
+		my($value,$label,$help) = @$_;
 		encode($label, $ESCAPE_CHARS::std) unless $no_encode;
+		encode($help, $ESCAPE_CHARS::std) if $help;
 		if($value =~ /^\s*\~\~(.*)\~\~\s*$/) {
 			my $lab = $1;
 			$lab =~ s/"/&quot;/g;
@@ -872,7 +953,7 @@ sub box {
 		$value = ''     if ! length($value);
 		$label = $value if ! length($label);
 
-		$run .= '<TR>' if $inc && ! ($i % $inc);
+		$run .= '<tr>' if $inc && ! ($i % $inc);
 		$i++;
 
 		undef $opt->{selected};
@@ -913,8 +994,10 @@ sub box {
 			$opt->{tlabel} = $label;
 		}
 
+		$opt->{ttitle} = $help;
+
 		$run .= attr_list($template, $opt);
-		$run .= '</TR>' if $inc && ! ($i % $inc);
+		$run .= '</tr>' if $inc && ! ($i % $inc);
 	}
 	$run .= $footer;
 }
@@ -932,7 +1015,7 @@ sub options_to_array {
 	my @out;
 
 	if($passed =~ m{^[^=]*\0}) {
-		$passed = filter_value($passed, 'option_format');
+		$passed = Vend::Interpolate::filter_value($passed, 'option_format');
 	}
 
 	my $delim = $opt->{delimiter} || ',';
@@ -1045,6 +1128,18 @@ if($opt->{debug}) {
 	$opt->{cols}      ||= $opt->{width} || $opt->{size};
 	$opt->{rows}      ||= $opt->{height};
 
+	if($opt->{js_check}) {
+		my @checks = grep /\w/, split /[\s,\0]+/, $opt->{js_check};
+		for(@checks) {
+			if(my $sub = Vend::Util::codedef_routine('JavaScriptCheck', $_)) {
+				$sub->($opt);
+			}
+			else {
+				::logError('Unknown %s: %s', 'JavaScriptCheck', $_);
+			}
+		}
+	}
+
 	# This handles the embedded attribute information in certain types,
 	# for example: 
 	# 
@@ -1212,68 +1307,54 @@ if($opt->{debug}) {
 	}
 
     $opt->{encoded} = encode($opt->{value}, $ESCAPE_CHARS::std);
+	if($opt->{display_filter}) {
+		my $newv = Vend::Interpolate::filter_value(
+								$opt->{display_filter},
+								$opt->{value},
+							);
+		$opt->{filtered} = encode($newv, $ESCAPE_CHARS::std);
+	}
     $opt->{value} =~ s/&#91;/\[/g if $opt->{enable_itl};
 
 	if($opt->{class}) {
-		$opt->{extra}	= $opt->{extra}
-						? qq{$opt->{extra} class="$opt->{class}"}
-						: qq{class="$opt->{class}"}
-						;
-	}
-
-	# Action taken for various types
-	my %daction = (
-		checkbox    => \&box,
-		combo		=> \&combo,
-		date		=> \&date_widget,
-		default     => \&template_sub,
-		display     => \&current_label,
-		links		=> \&links,
-		movecombo	=> \&movecombo,
-		multiple    => \&dropdown,
-		noyes		=> \&noyes,
-		option_format => \&option_widget,
-		options     => \&show_options,
-		labels      => \&show_labels,
-		radio       => \&box,
-		select      => \&dropdown,
-		show        => \&show_data,
-		value       => sub { my $opt = shift; return $opt->{encoded} },
-		realvalue   => sub { my $opt = shift; return $opt->{value} },
-		yesno		=> \&yesno,
-	);
-
-	## The user/admin widget space
-	# Optimization for large lists
-	unless($Vend::UserWidget) {
-		my $ref;
-		$Vend::UserWidget	= ($ref = $Vend::Cfg->{CodeDef}{Widget})
-							? $ref->{Routine}
-							: {};
-		if(my $ref = $Global::CodeDef->{Widget}{Routine}) {
-			while ( my ($k, $v) = each %$ref) {
-				next if $Vend::UserWidget->{$k};
-				$Vend::UserWidget->{$k} = $v;
-			}
+		if($opt->{extra}) {
+			$opt->{extra} =~ s{(^|\s+)class=(["'])?[^\s'"]+\2}{$1};
+			$opt->{extra} =~ s/\s+$//;
+			$opt->{extra} .=	qq{class="$opt->{class}"} ;
+		}
+		else {
+			$opt->{extra} =	qq{class="$opt->{class}"} ;
 		}
 	}
 
-	my $sub =  $Vend::UserWidget->{$type}
-			|| $daction{$type}
-			|| $daction{default};
+	# Optimization for large lists, we cache the widgets
+	$Vend::UserWidget ||= Vend::Config::map_widgets();
+
+	my $sub =  $Vend::UserWidget->{$type};
+	if(! $sub and $Global::AccumulateCode) {
+		$sub = Vend::Config::code_from_file('Widget', $type)
+			and $Vend::UserWidget->{$type} = $sub;
+	}
+
+	# Last in case "default" widget is removed
+	$sub ||= $Vend::UserWidget->{default} || \&template_sub;
 
 	if($opt->{variant}) {
 #::logDebug("variant='$opt->{variant}'");
 		$opt->{subwidget}	=  $Vend::UserWidget->{$opt->{variant}}
-							|| $daction{$opt->{variant}}
-							|| $daction{default};
+							||  $Vend::UserWidget->{default};
 	}
 
 	if(my $c = $opt->{check}) {
 		$c = "$opt->{name}=$c" unless $c =~ /=/;
 		HTML::Entities::encode($c);
-		no warnings;
-		$opt->{append} .= qq{<input type=hidden name="mv_individual_profile" value="$c">};
+		$opt->{append} .= qq{<input type="hidden" name="mv_individual_profile" value="$c">};
+	}
+
+	if($opt->{js}) {
+		$opt->{extra} ||= '';
+		$opt->{extra} .= " $opt->{js}";
+		$opt->{extra} =~ s/^\s+//;
 	}
 	return $sub->($opt, $data);
 }
@@ -1285,7 +1366,9 @@ sub parse_type {
 		return $opt;
 	}
 
+	my %alias = (qw/ datetime date_time /);
 	my $type = $opt->{type} = lc($opt->{type}) || 'text';
+	$type = $alias{$type} if $alias{$type};
 	return $type if $type =~ /^[a-z][a-z0-9]*$/;
 
 	if($type =~ /^text/i) {
@@ -1305,17 +1388,30 @@ sub parse_type {
 			$opt->{type} = 'text';
 		}
 	}
-	elsif($type =~ /^date_?(time([^_]*))?(_?blank)?/i) {
+	elsif($type =~ /^date(.*)/i) {
 		$opt->{type} = 'date';
-		$opt->{blank} = 1 if $3;
-		if ($1) {
-			my $extra = $2;
-			$opt->{time} = 1;
-			$opt->{ampm} = 1
-				if $extra =~ /ampm/i;
+		my $extra = $1;
+		if ($extra) {
+			$opt->{time} = 1 if $extra =~ /time/i;
+			$opt->{ampm} = 1 if $extra =~ /ampm/i;
+			$opt->{blank} = 1 if $extra =~ /blank/i;
+			($extra =~ /\(\s*(\s*\d+\s*(,\s*\d+\s*)+)\s*\)/i
+					and $opt->{minutes} = $1)
+			  or
+			($extra =~ /half/i and $opt->{minutes} = 'half_hourly') 
+			  or 
+			($extra =~ /hourly/i and $opt->{minutes} = 'hourly')
+			  or 
+			($extra =~ /tens/i and $opt->{minutes} = 'tens')
+			;
+			if($extra =~ s/(\d+)-(\d+)//) {
+				$opt->{start_hour} = $1;
+				$opt->{end_hour} = $2;
+			}
 			$opt->{time_adjust} = $1
 				if $extra =~ /([+-]?\d+)/i;
 		}
+#::logDebug("minutes=$opt->{minutes}");
 	}
 	elsif($type =~ /^hidden_text/i) {
 		$opt->{type} = 'hiddentext';
@@ -1376,10 +1472,15 @@ sub parse_type {
 		$opt->{type} = 'combo';
 		$opt->{reverse} = 1;
 	}
-	elsif($type =~ /^move_combo[ _]*(?:(\d+)(?:[ _]+(\d+))?)?/i) {
-		$opt->{rows} = $opt->{rows} || $1 || 1;
-		$opt->{cols} = $opt->{cols} || $2 || 16;
+	elsif($type =~ /^links_*nbsp/i) {
+		$opt->{nbsp} = 1;
+		$opt->{type} = 'links';
+	}
+	elsif($type =~ /^move_*combo[ _]*(?:(\d+)(?:[ _]+(\d+))?)?/i) {
+		$opt->{rows} = $opt->{rows} || $opt->{height} || $1 || 1;
+		$opt->{cols} = $opt->{cols} || $opt->{width} || $2 || 16;
 		$opt->{type} = 'movecombo';
+		$opt->{replace} = 1 if $type =~ /replace/;
 	}
 	elsif($type =~ /multi/i) {
 		$opt->{type} = 'select';
