@@ -1,10 +1,10 @@
 # Vend::Glimpse - Search indexes with Glimpse
 #
-# $Id: Glimpse.pm,v 2.9 2003/06/18 17:34:44 jon Exp $
+# $Id: Glimpse.pm,v 2.14 2005/11/08 18:14:45 jon Exp $
 #
 # Adapted for use with Interchange from Search::Glimpse
 #
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -26,8 +26,11 @@ package Vend::Glimpse;
 require Vend::Search;
 @ISA = qw(Vend::Search);
 
-$VERSION = substr(q$Revision: 2.9 $, 10);
+$VERSION = substr(q$Revision: 2.14 $, 10);
 use strict;
+use Vend::File;
+use Vend::Util;
+
 
 sub array {
 	my ($s, $opt) = @_;
@@ -119,6 +122,21 @@ sub search {
 			unless Vend::Util::file_name_is_absolute($_);
 	}
 
+	unless ($s->{mv_no_hide} or $s->{mv_hide_field}) {
+		my $dbref = $s->{table} || undef;
+		if (! $dbref) {
+			my $table = $s->{mv_field_file};
+			$table =~ s:.*/::;
+			$table =~ s/\..*//;
+			$dbref = Vend::Data::database_exists_ref($table);
+		}
+		if ($dbref) {
+			my $hf = $dbref->config('HIDE_FIELD');
+			$s->{mv_hide_field} = $hf if defined $hf;
+#::logDebug("mv_hide_field=$hf");
+		}
+	}
+
 #::logDebug("gsearch: self=" . ::Vend::Util::uneval_it({%$s}));
 	$s->{mv_return_delim} = $s->{mv_index_delim}
 		unless defined $s->{mv_return_delim};
@@ -149,7 +167,7 @@ sub search {
 	push(@cmd, "-F '$s->{mv_search_file}[0]'")
 		if defined $s->{mv_search_file}[0];
 
-	push(@cmd, '-w') unless $s->{mv_substring_match};
+	push @cmd, '-w' unless @{$s->{mv_substring_match}};
 	push(@cmd, '-O -X') if $s->{mv_return_file_name};
 	
 	if($s->{mv_return_file_name}) { 
@@ -324,7 +342,7 @@ EOF
 		$s->hash_fields($s->{mv_field_names}, qw/mv_sort_field/);
 #::logDebug("gsearch after hash fields: self=" . ::Vend::Util::uneval_it({%$s}));
 		$s->sort_search_return(\@out);
-		$delayed_return = $s->get_return(1,1);
+		$delayed_return = $s->get_return(1);
 		@out = map { $delayed_return->($_) } @out;
 	}
 #::logDebug("after delayed return: self=" . ::Vend::Util::uneval_it({%$s}));
@@ -356,7 +374,7 @@ EOF
 		@out = map { join $s->{mv_return_delim}, @$_ } @out;
 		$s->{mv_results} = join $s->{mv_record_delim}, @out;
 	}
-	else {
+	elsif($s->{mv_return_reference} eq 'HASH') {
 		my $col = scalar @{$s->{mv_return_fields}};
 		my @col;
 		my @names;
@@ -375,29 +393,26 @@ EOF
 }
 
 # Unfortunate hack need for Safe searches
-*escape         	= \&Vend::Search::escape;
-*spec_check         = \&Vend::Search::spec_check;
-*get_scalar         = \&Vend::Search::get_scalar;
-*more_matches       = \&Vend::Search::more_matches;
-*get_return         = \&Vend::Search::get_return;
-*map_ops            = \&Vend::Search::map_ops;
-*get_limit          = \&Vend::Search::get_limit;
-*saved_params       = \&Vend::Search::saved_params;
-*range_check        = \&Vend::Search::range_check;
 *create_search_and  = \&Vend::Search::create_search_and;
 *create_search_or   = \&Vend::Search::create_search_or;
-*save_context       = \&Vend::Search::save_context;
 *dump_options       = \&Vend::Search::dump_options;
+*escape             = \&Vend::Search::escape;
+*get_limit          = \&Vend::Search::get_limit;
+*get_return         = \&Vend::Search::get_return;
+*get_scalar         = \&Vend::Search::get_scalar;
+*hash_fields        = \&Vend::Search::hash_fields;
+*map_ops            = \&Vend::Search::map_ops;
+*more_matches       = \&Vend::Search::more_matches;
+*range_check        = \&Vend::Search::range_check;
+*restore_specs      = \&Vend::Search::restore_specs;
+*save_context       = \&Vend::Search::save_context;
 *save_more          = \&Vend::Search::save_more;
+*save_specs         = \&Vend::Search::save_specs;
+*saved_params       = \&Vend::Search::saved_params;
+*search_error       = \&Vend::Search::search_error;
 *sort_search_return = \&Vend::Search::sort_search_return;
-*get_scalar 		= \&Vend::Search::get_scalar;
-*hash_fields 		= \&Vend::Search::hash_fields;
-*save_specs 		= \&Vend::Search::save_specs;
-*restore_specs 		= \&Vend::Search::restore_specs;
-*splice_specs 		= \&Vend::Search::splice_specs;
-*search_error 		= \&Vend::Search::search_error;
-*save_more 			= \&Vend::Search::save_more;
-*sort_search_return = \&Vend::Search::sort_search_return;
+*spec_check         = \&Vend::Search::spec_check;
+*splice_specs       = \&Vend::Search::splice_specs;
 
 1;
 __END__
