@@ -1,6 +1,6 @@
 # Vend::Order - Interchange order routing routines
 #
-# $Id: Order.pm,v 2.83.2.1 2005/12/07 17:15:49 kwalsh Exp $
+# $Id: Order.pm,v 2.83.2.4 2007/01/31 00:13:36 pajamian Exp $
 #
 # Copyright (C) 2002-2003 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
@@ -29,7 +29,7 @@
 package Vend::Order;
 require Exporter;
 
-$VERSION = substr(q$Revision: 2.83.2.1 $, 10);
+$VERSION = substr(q$Revision: 2.83.2.4 $, 10);
 
 @ISA = qw(Exporter);
 
@@ -1022,6 +1022,10 @@ sub check_order_each {
 		s/^\s+//;
 		s/\s+$//;
 		($val, $var, $message) = do_check($_, $vref);
+
+		# no actual check on this line, skip to next
+		next if /^&(?:and|or)\s*$/i;
+
 		if(defined $And) {
 			if($And) {
 				$val = ($last_one && $val);
@@ -1754,9 +1758,11 @@ sub route_order {
 
 		my $use_mime;
 		undef $::Instance->{MIME};
-		if(not ($route->{credit_card} || $route->{encrypt}) ) {
-			$::Values->{mv_credit_card_info}
-				=~ s/^(\s*\w+\s+)(\d\d)[\d ]+(\d\d\d\d)/$1$2 NEED ENCRYPTION $3/;
+		if(not ($pre_encrypted || $route->{credit_card} || $route->{encrypt}) ) {
+		    unless ($::Values->{mv_credit_card_info}
+			    =~ s/^(\s*\w+\s+)(\d\d)[\d ]+(\d\d\d\d.*?)(?:\s+\d{3,4})?$/$1$2 NEED ENCRYPTION $3/) {
+			$::Values->{mv_credit_card_info} = 'NEED ENCRYPTION';
+		    }
 		}
 		eval {
 			$page = interpolate_html($page) if $page;
