@@ -1,8 +1,8 @@
 # Vend::Payment::BoA - Interchange BoA support
 #
-# $Id: BoA.pm,v 1.10 2003/08/04 22:01:08 racke Exp $
+# $Id: BoA.pm,v 1.13 2005/11/08 18:14:46 jon Exp $
 #
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 1999-2002 Red Hat, Inc.
 #
 # by Mark Johnson based off of AuthorizeNet.pm by
@@ -279,7 +279,7 @@ BEGIN {
 	}
 
 	::logGlobal("%s payment module initialized, using %s", __PACKAGE__, $selected)
-		unless $Vend::Quiet;
+		unless $Vend::Quiet or ! $Global::VendRoot;
 
 }
 
@@ -457,7 +457,7 @@ sub boa {
 #::logDebug("BoA post_data response: " . ::uneval($thing) );
 	
 	my %result;
-	my $sep = "<BR>";
+	my $sep = "<br>";
 
 	$page =~ s/\r*<html>.*$//is;
 
