@@ -1,7 +1,8 @@
 # Vend::Payment::iTransact - Interchange iTransact Support
 #
-# $Id: iTransact.pm,v 2.4 2003/08/04 22:01:09 racke Exp $
+# $Id: iTransact.pm,v 2.6 2005/11/08 18:14:46 jon Exp $
 #
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 1999-2002 Red Hat, Inc.
 #
 # Written by Cameron Prince and Mark Johnson, based on code by Mike Heins.
@@ -209,7 +210,7 @@ BEGIN {
 	}
 
 	::logGlobal("%s payment module initialized, using %s", __PACKAGE__, $selected)
-		unless $Vend::Quiet;
+		unless $Vend::Quiet or ! $Global::VendRoot;
 
 }
 
