@@ -1,8 +1,8 @@
 # Vend::Scan - Prepare searches for Interchange
 #
-# $Id: Scan.pm,v 2.25 2003/07/30 04:00:45 mheins Exp $
+# $Id: Scan.pm,v 2.31 2005/11/08 18:14:45 jon Exp $
 #
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -30,9 +30,10 @@ require Exporter;
 			perform_search
 			);
 
-$VERSION = substr(q$Revision: 2.25 $, 10);
+$VERSION = substr(q$Revision: 2.31 $, 10);
 
 use strict;
+no warnings qw(uninitialized numeric);
 use Vend::Util;
 use Vend::File;
 use Vend::SQL_Parser;
@@ -107,8 +108,10 @@ my @Order = ( qw(
 	mv_unique
 	mv_more_matches
 	mv_value
+	mv_no_more
 	mv_next_search
 	mv_search_reference
+	mv_more_permanent
 	prefix
 ));
 
@@ -157,12 +160,14 @@ my %Scan = ( qw(
 	ne  mv_negate
 	ng  mv_negate
 	nh  mv_no_hide
+	nm  mv_no_more
 	np  mv_nextpage
 	ns  mv_next_search
 	nu  mv_numeric
 	op  mv_column_op
 	os  mv_orsearch
 	pf  prefix
+	pm  mv_more_permanent
 	ra  mv_return_all
 	rd  mv_return_delim
 	re	mv_search_reference
@@ -208,7 +213,7 @@ my %Parse = (
 	mv_column_op            =>  \&_array,
 	mv_coordinate           =>  \&_yes,
 	mv_no_hide              =>  \&_yes,
-
+	mv_no_more              =>  \&_yes,
 	mv_field_names          =>	\&_array,
 	mv_spelling_errors      => 	sub { my $n = int($_[1]); $n < 8 ? $n : 1; },
 	mv_dict_limit           =>  \&_dict_limit,
@@ -261,6 +266,10 @@ sub create_last_search {
 			push @out, "$RevScan{$key}=$_";
 		}
 	}
+
+	# Make repeatable for permanent store
+	@out = sort @out;
+
 	$Vend::Session->{last_search} = join "/", 'scan', @out;
 }
 
@@ -409,12 +418,15 @@ sub perform_search {
 	my($c,$more_matches,$pre_made) = @_;
 #::logDebug('searching....');
 	if (!$c) {
+#::logDebug("No search object");
 		return undef unless $Vend::Session->{search_params};
 		($c, $more_matches) = @{$Vend::Session->{search_params}};
 		unless($c->{mv_cache_key}) {
+#::logDebug("No cache key");
 			Vend::Scan::create_last_search($c);
 			$c->{mv_cache_key} = generate_key($Vend::Session->{last_search});
 		}
+#::logDebug("Found search object=" . ::uneval($c));
 	}
 	elsif ($c->{mv_search_immediate}) {
 		unless($c->{mv_cache_key}) {
@@ -434,11 +446,13 @@ sub perform_search {
 	my %options;
 	$options{mv_session_id} = $c->{mv_session_id} || $Vend::SessionID;
 	if($c->{mv_more_matches}) {
-		@options{qw/mv_cache_key mv_next_pointer mv_last_pointer mv_matchlimit/}
+#::logDebug("Found search object=" . ::uneval($c));
+		@options{qw/mv_cache_key mv_next_pointer mv_last_pointer mv_matchlimit mv_more_permanent/}
 			= split /:/, $c->{mv_more_matches};
 		$options{mv_more_id} = $c->{mv_more_id}
 			if $c->{mv_more_id};
 		my $s = new Vend::Search %options;
+#::logDebug("resulting search object=" . ::uneval($s));
 		$q = $s->more_matches();
 		finish_search($q);
 		return $q;
