@@ -1,8 +1,8 @@
 # Vend::Search - Base class for search engines
 #
-# $Id: Search.pm,v 2.22 2004/04/08 15:05:09 mheins Exp $
+# $Id: Search.pm,v 2.30 2005/04/30 15:09:58 mheins Exp $
 #
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2004 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -22,9 +22,10 @@
 
 package Vend::Search;
 
-$VERSION = substr(q$Revision: 2.22 $, 10);
+$VERSION = substr(q$Revision: 2.30 $, 10);
 
 use strict;
+no warnings qw(uninitialized numeric);
 use vars qw($VERSION);
 
 sub new {
@@ -347,17 +348,50 @@ sub more_matches {
 	my($s) = @_;
 	$s->{more_in_progress} = 1;
 
-	my $id = $s->{mv_more_id} || $s->{mv_session_id};
-	$id .= ".$s->{mv_cache_key}";
+	my $id;
+	if($s->{mv_more_permanent}) {
+#::logDebug("Permanent more");
+		$id = $s->{mv_cache_key};
+	}
+	else {
+		$id = $s->{mv_more_id} || $s->{mv_session_id};
+		$id .= ".$s->{mv_cache_key}";
+	}
 	
-	my $file = Vend::Util::get_filename($id);
-#::logDebug("more_matches: $id from $file");
-
+	my $file;
 	my $obj;
 	eval {
-		$obj = Vend::Util::eval_file($file);
+		if($Vend::Cfg->{MoreDB}) {
+#::logDebug("more_matches: $id from $Vend::Cfg->{SessionDB}");
+			eval {
+				my $db = Vend::Util::dbref($Vend::Cfg->{SessionDB});
+				$obj = Vend::Util::evalr( $db->field($id,'session') );
+			};
+			$@ and return $s->search_error(
+							"Object saved wrong in session DB for search ID %s.",
+							$id,
+						);
+		}
+		else {
+			if($s->{mv_more_permanent}) {
+				$file = Vend::File::get_filename($id, 2, 1, $Vend::Cfg->{PermanentDir});
+			}
+			else {
+				$file = Vend::File::get_filename($id);
+			}
+
+#::logDebug("more_matches: $id from $file");
+			eval {
+				$obj = Vend::Util::eval_file($file);
+			};
+			$@ and return $s->search_error(
+							"Object saved wrong in %s for search ID %s.",
+							$file,
+							$id,
+						);
+		}
 	};
-	$@ and return $s->search_error("Object saved wrong in $file for search ID $id.");
+
 	for(qw/mv_cache_key mv_matchlimit /) {
 		$obj->{$_} = $s->{$_};
 	}
@@ -521,6 +555,9 @@ sub create_text_query {
 			? $s->{mv_substring_match}[$i]
 			: $s->{mv_substring_match};
 
+	$string =~ s/(\w)'(s)\b/$1$2/g;
+	$string =~ s/(\w)'(\w+)/$1 and $2/g;
+	$string =~ s/'//g;
 #::logDebug("query creation called, op=$op cs=$cs ac=$ac");
 	if($op eq 'aq') {
 		$q = new Text::Query($string,
@@ -612,13 +649,12 @@ sub map_ops {
 		$c->[$i] =~ tr/ \t//;
 		my $o = $c->[$i];
 		$c->[$i] = $s->{mv_numeric}[$i]
-				? $numopmap{$o}
-				: $stropmap{$o};
+				? [ @{$numopmap{$o}} ]
+				: [ @{$stropmap{$o}} ];
 		if(! $c->[$i]) {
 			my $r;
 			$c->[$i] = [$r, $o], next
-				if  $r = $Global::CodeDef->{SearchOp}
-				and $r = $r->{Routine}{$o};
+				if  $r = Vend::Util::codedef_routine('SearchOp',$o);
 		}
 	}
 	@{$s->{mv_column_op}};
@@ -662,6 +698,22 @@ sub get_limit {
 		$code       = "sub {\nmy \$line = shift;\n";
 	}
 	$code .= "my \@fields = \@\$line;\n";
+
+	my $have_hf;
+	if ($s->{mv_hide_field}) {
+		$s->{mv_field_hash} = create_field_hash($s) 
+			unless $s->{mv_field_hash};
+		my $hf = $s->{mv_field_hash}{$s->{mv_hide_field}};
+		if (defined $hf) {
+			$code .= "return if \$fields[$hf];\n";
+			$have_hf = 1;
+		}
+		else {
+		 	::logError("Ignoring unknown mv_hide_field specification: $s->{mv_hide_field}");
+			delete $s->{mv_hide_field};
+		}
+	}
+		
 	my $join_key;
 	$join_key = defined $s->{mv_return_fields} ? $s->{mv_return_fields}[0] : 0;
 	$join_key = 0 if $join_key eq '*';
@@ -937,7 +989,7 @@ EOF
 		}
 		 $code .= $range_code;
 		 $code .= <<EOF;
-	local(\$_) = join q{$s->{mv_index_delim}}, \@\$line[$fields];
+	local(\$_) = join q{$s->{mv_index_delim}}, \@fields[$fields];
 	return(1) if &\$sub();
 	return undef;
 }
@@ -949,6 +1001,12 @@ EOF
 	$range_code
 	$ender
 }
+EOF
+	}
+	elsif ($have_hf) {
+		$code .= <<'EOF';
+return 1;
+}
 EOF
 	}
 	# If there is to be no limit_sub
@@ -1114,11 +1172,23 @@ sub search_error {
 
 sub save_more {
 	my($s, $out) = @_;
+	return 1 if $s->{mv_no_more};
 	return if $MVSAFE::Safe;
 	my $file;
 	delete $s->{dbref} if defined $s->{dbref};
-	my $id = $s->{mv_more_id} || $Vend::SessionID;
-	$id .= ".$s->{mv_cache_key}";
+
+	my $id;
+	my $storedir;
+
+	unless($s->{mv_more_permanent}) {
+		$id = $s->{mv_more_id} || $Vend::SessionID;
+		$id .= ".$s->{mv_cache_key}";
+	}
+	else {
+		$id = $s->{mv_cache_key};
+		$storedir = $Vend::Cfg->{PermanentDir};
+	}
+
 	if ($s->{matches} > $s->{mv_matchlimit} and $s->{mv_matchlimit} > 0) {
 		$s->{overflow} = 1;
 		$s->{mv_next_pointer} = $s->{mv_matchlimit};
@@ -1134,15 +1204,30 @@ sub save_more {
 		more_alpha($s,$out);
 	}
 	
-	$file = Vend::Util::get_filename($id); 
-#::logDebug("save_more: $id to $file.");
 	my $new = { %$s };
-	$new->{mv_results} = $out;
 	delete $new->{search_routines};
+	$new->{mv_results} = $out;
+
+	if($Vend::Cfg->{MoreDB}) {
+#::logDebug("save_more: $id to Session DB.");
 #::logDebug("save_more:object:" . ::uneval($new));
-	eval {
-		Vend::Util::uneval_file($new, $file);
-	};
+		my $db = Vend::Util::dbref($Vend::Cfg->{SessionDB});
+		$db->set_field($id, 'session', Vend::Util::uneval_fast($new));
+	}
+	else {
+#::logDebug("save_more: $id to $file.");
+#::logDebug("save_more:object:" . ::uneval($new));
+		if($storedir) {
+			$file = Vend::File::get_filename($id,2,1,$storedir); 
+		}
+		else {
+			$file = Vend::File::get_filename($id); 
+		}
+
+		eval {
+			Vend::Util::uneval_file($new, $file);
+		};
+	}
 	$@ and return $s->search_error("failed to store more matches");
 	return 1;
 }
