@@ -1,6 +1,6 @@
 # Vend::Session - Interchange session routines
 #
-# $Id: Session.pm,v 2.16 2004/02/24 20:53:58 mheins Exp $
+# $Id: Session.pm,v 2.23 2005/04/18 18:57:32 mheins Exp $
 # 
 # Copyright (C) 2002-2003 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
@@ -27,7 +27,7 @@ package Vend::Session;
 require Exporter;
 
 use vars qw($VERSION);
-$VERSION = substr(q$Revision: 2.16 $, 10);
+$VERSION = substr(q$Revision: 2.23 $, 10);
 
 @ISA = qw(Exporter);
 
@@ -202,8 +202,13 @@ sub count_ip {
 	mkdir $dir, 0777 unless -d $dir;
 	my $fn = Vend::Util::get_filename($ip, 2, 1, $dir);
 	if(-f $fn) {
-		my $grace = time() - ($Global::Variable->{MV_ROBOT_EXPIRE} || 86400);
-		unlink $fn if -M $fn < $grace;
+		my $grace = $Vend::Cfg->{Limit}{robot_expire} || 1;
+		my @st = stat(_);
+		my $mtime = (time() - $st[9]) / 86400;
+		if($mtime > $grace) {
+			::logDebug("ip $ip allowed back in due to '$mtime' > '$grace' days");
+			unlink $fn;
+		}
 	}
 	return Vend::CounterFile->new($fn)->inc() if $inc;
 	return Vend::CounterFile->new($fn)->value();
@@ -247,14 +252,16 @@ sub new_session {
 		}
 		$name = session_name();
 		unless ($File_sessions) { 
+			lock_session($name);
 			last unless defined $Vend::SessionDBM{$name};
+			unlock_session($name);
 		}
 		else {
 			last unless exists $Vend::SessionDBM{$name};
 		}
     }
 	$Vend::new_session = 1;
-	count_ip(1);
+	count_ip(1) if $Vend::Cfg->{RobotLimit};
 	undef $Vend::Cookie;
     $Vend::SessionName = $name;
     init_session();
@@ -302,9 +309,9 @@ sub write_session {
     $Vend::Session->{'time'} = $time;
 	delete $Vend::Session->{values}->{mv_credit_card_number};
     my $save = delete $Vend::Session->{'user'};
-	for(@Vend::TmpScratch) {
-		delete $::Scratch->{$_};
-	}
+
+	delete @{$::Scratch}{@Vend::TmpScratch};
+
 	$Vend::Session->{username} = $Vend::username;
 	$Vend::Session->{admin} = $Vend::admin;
 	$Vend::Session->{groups} = $Vend::groups;
@@ -319,20 +326,25 @@ sub write_session {
 
 sub unlock_session {
 #::logDebug ("unlock session id=$Vend::SessionID  name=$Vend::SessionName\n");
+	my $name = shift;
+	$name ||= $Vend::SessionName;
 	delete $Vend::SessionDBM{'LOCK_' . $Vend::SessionName}
 		unless $File_sessions;
 }
 
 sub lock_session {
 	return 1 if $File_sessions;
+	my $name = shift;
+	$name ||= $Vend::SessionName;
 #::logDebug ("lock session id=$Vend::SessionID  name=$Vend::SessionName\n");
-	my $lockname = 'LOCK_' . $Vend::SessionName;
+	my $lockname = "LOCK_$name";
 	my ($tried, $locktime, $sleepleft, $pid, $now, $left);
 	$tried = 0;
 
 	LOCKLOOP: {
-		if (defined $Vend::SessionDBM{$lockname}) {
-			($locktime, $pid) = split /:/, $Vend::SessionDBM{$lockname}, 2;
+		my $lv;
+		if (defined ($lv = $Vend::SessionDBM{$lockname}) ) {
+			($locktime, $pid) = split /:/, $lv, 2;
 		}
 		$now = time;
 		if(defined $locktime and $locktime) {
@@ -421,6 +433,7 @@ sub read_session {
     $::Values	= $Vend::Session->{values};
     $::Scratch	= $Vend::Session->{scratch};
     $::Carts	= $Vend::Session->{carts};
+    $::Discounts = $Vend::Session->{discount};
     $Vend::Interpolate::Tmp ||= {};
     $::Control	= $Vend::Interpolate::Tmp->{control} = [];
 	tie $Vend::Items, 'Vend::Cart';
@@ -475,6 +488,7 @@ sub init_session {
 		'values'	=> { %{$Vend::Cfg->{ValuesDefault}} },
 		'carts'		=> {main => []},
 		'levies'	=> {main => []},
+		'discount_space'	=> {main => {}},
     };
 	$Vend::Session->{shost} = $CGI::remote_addr
 		if $CGI::secure;
@@ -485,6 +499,15 @@ sub init_session {
 	tie $Vend::Items, 'Vend::Cart';
 	$::Values->{mv_shipmode} = $Vend::Cfg->{DefaultShipping}
 		if ! defined $::Values->{mv_shipmode};
+	$::Discounts
+		= $Vend::Session->{discount}
+		= $Vend::Session->{discount_space}{main};
+	if(my $macro = $Vend::Cfg->{SpecialSub}{init_session}) {
+		Vend::Dispatch::run_macro(
+				$macro,
+				$Vend::Session,
+			);
+	}
 }
 
 sub dump_sessions {
