@@ -1,8 +1,8 @@
 # Vend::Table::LDAP - Interchange LDAP pseudo-table access
 #
-# $Id: LDAP.pm,v 2.10 2003/07/14 09:17:51 racke Exp $
+# $Id: LDAP.pm,v 2.13 2005/11/08 18:14:48 jon Exp $
 #
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -25,7 +25,7 @@
 
 package Vend::Table::LDAP;
 @ISA = qw/Vend::Table::Common/;
-$VERSION = substr(q$Revision: 2.10 $, 10);
+$VERSION = substr(q$Revision: 2.13 $, 10);
 use strict;
 
 use vars qw(
@@ -514,7 +514,6 @@ sub query {
 #::logDebug("receieved query. object=" . ::uneval($opt));
 
 	if(defined $opt->{values}) {
-		# do nothing
 		@arg = $opt->{values} =~ /['"]/
 				? ( Text::ParseWords::shellwords($opt->{values})  )
 				: (grep /\S/, split /\s+/, $opt->{values});
@@ -670,6 +669,13 @@ eval {
 		$return = $opt->{failure} || undef;
 	}
 
+	if($opt->{search_label}) {
+		$::Instance->{SearchObject}{$opt->{search_label}} = {
+			mv_results => $ref,
+			mv_field_names => \@na,
+		};
+	}
+
 	return scalar @{$ref || []}
 		if $opt->{row_count};
 	return Vend::Interpolate::region($opt, $text)
